#!/usr/bin/perl5

# README
# ---------------------------------------------------
# You shouldn't need to modify anything in here. Please
# be sure to read the Readme that is provided 
# (Plugins > Recommend_It > Readme). 
#

# COPYRIGHT
# ---------------------------------------------------
# 1984-2007. All Rights Reserved. UltraNerds.com.
# Illegal use of this Plugin, in any shape or form,
# is strictly forbidden. UltraNerds.com do not accept 
# ANY responsability for the use of this plugin.

#####################################################
# Load required modules.
# ---------------------------------------------------
    use strict;
    use lib '';			                      
    use Links qw/$DB $IN $USER $CFG/;
    use Links::SiteHTML;
    use GT::Mail;
    use CGI::Carp qw(fatalsToBrowser); 
    use Links::Plugins;
    use Links;
    use vars qw/$REC_CFG/;

    Links::init();
    Links::init_user();

# Need to put the configuration variables into the $CFG hash, or similar, and then 
# they are globally available, and can be assigned/passed in a hash between routines.
# Works with persistence -- mod_perl speedyCGI, etc
    $REC_CFG  = Links::Plugins::get_plugin_user_cfg ('Recommend_It');

# Setup our values grabbed from the plugin setting...
    my $RequireLogin = $REC_CFG->{'UserRequired'};

# Now we need to work out the full script name.
    $REC_CFG->{'ScriptURL'} = $CFG->{db_cgi_url} . "/";
     
# print out our header...
# only show if we require a login, and their $USER value 
# doesn't exist.. otherwise CGI.pm redirect() will not work, as
# a header has already been passed in....
   if (!$RequireLogin)          { print $IN->header(); }

# lets initiate the script now...
   &main();


sub main { 

# Define the variables
# ---------------------------------------------------
	my ($id, $db_links, $rec, $SendList);

 # only ask them to login if the admin has set this to 'Yes' in the setup...
  if ($RequireLogin == 1) {
	   if (!$USER) {
          print $IN->redirect( Links::redirect_login_url ('add') );
          exit;
       }      
   }  

   # print our header now, as we are safe to print content to the browser..
    print $IN->header();

	# Get the Links ID number from the input.
	$id  = $IN->param('ID');

	# check to see if an ID was submitted and that it was numeric
    if (!(defined $id) or !($id =~ /^\d+$/)) {
	      
		  print Links::SiteHTML::display('error', {error => "Invalid id: $id"});
		  exit;
    } 

	# Check to see if the ID/Link record exists
	  $db_links   = $DB->table('Links'); 	# first grab a new db handle
	
	  $rec = $db_links->get ($id);	# see if the ID record exists

  # show an error if the link they are trying to refer is not valid...
	  if (!$rec) {	  
		  	print Links::SiteHTML::display ('error', { error => Links::language ('JUMP_INVALIDID', $id) });
		  	exit;			
	  }

  # pass the script URL, so we can access this tag in the templates etc....
    $rec->{'ScriptURL'} = $REC_CFG->{'ScriptURL'};


  # now lets decide if we want to show the form, or actually do the sending...
  # here we show the form
    if (!$IN->param('action')) {

         print Links::SiteHTML::display('recommend_it_form', $rec);
         exit;

  # and down here is where the magic goes on , and the emails etc are sent...      
    } else {

      # work out how many people we should be sending too... 
       my $number_of_users = $IN->param('number_of_fields');
        
       for (my $i = 1; $i <= $number_of_users; $i++) {

          # skip this record if they don't need to be sent too...
           if (!$IN->param("recipemail_$i")) { next; } 

          # first of all, lets check how valid the email address is...
           if (!$IN->param("recipemail_$i") || valid_address($IN->param("recipemail_$i")) == 0) {
             $rec->{'error'} .= "The email address for person $i doesn't seem to be valid....";
             print Links::SiteHTML::display('recommend_it_form', { %$rec, %$IN });  
             exit;
           }

          # now lets check how valid the name is...
           if (!$IN->param("recipname_$i") || length($IN->param("recipname_$i")) < 3) {
             $rec->{'error'} .= "The name you entered for person $i doesn't seem to be valid (or is empty)....";
             print Links::SiteHTML::display('recommend_it_form', { %$rec, %$IN });  
             exit;
           }
          
          # get the list ready, so we can put it in the stats table...
           $SendList .= $IN->param("recipname_$i") . " - " . $IN->param("recipemail_$i") . "<BR>";
           
           #print "Would have sent to " . $IN->param("recipname_$i") . "(" . $IN->param("recipemail_$i") . ") <BR>";

        }  
        
          # now lets check how valid the name is...
           if (!$IN->param("sendername") || length($IN->param("sendername")) < 3) {
             $rec->{'error'} .= "The name you entered for yourslef doesn't seem to be valid (or is empty)....";
             print Links::SiteHTML::display('recommend_it_form', { %$rec, %$IN });  
             exit;
           }

          # first of all, lets check how valid the email address is...
           if (!$IN->param("senderemail") || valid_address($IN->param("senderemail")) == 0) {
             $rec->{'error'} .= "The email address you entered for yourself, doesn't seem to be valid....";
             print Links::SiteHTML::display('recommend_it_form', { %$rec, %$IN });  
             exit;
           }       
           
          # get the date, so we can track stuff... 
           use GT::Date;
           my $now_time = GT::Date::date_get(); 
           
           $db_links->update( { 'Recommend_Count' => \"Recommend_Count + 1" }, { ID => $IN->param('ID') } ) || die $GT::SQL::error;

           my $stats_hash;
            $stats_hash->{'IP'} = $ENV{'REMOTE_ADDR'};
            $stats_hash->{'SenderEmail'} = $IN->param("senderemail"); 
            $stats_hash->{'SenderName'} = $IN->param("sendername"); 
            $stats_hash->{'SentList'} = $SendList;
            $stats_hash->{'Date'} = $now_time;
            
            $DB->table('Rec_Stats')->add( $stats_hash ) || die $GT::SQL::error;
           
          # now we need to actually send the emails....    
            &do_sending;

          # print out the success template...        
            print Links::SiteHTML::display('recommend_it_sent', { %$rec, %$IN, $SendList }); 

    }    
    
    
    
} # end the 'main' sub routine...

# ==============================================================================
# Send the actual emails....
# ==============================================================================
sub do_sending {

      # get GT::Mail imported...
       require GT::Mail;
       $GT::Mail::error ||= ''; # Silence -w

      # grab the links variables...
       my $rec = $DB->table('Links')->get ($IN->param('ID'));

      my ($SendList);

      # work out how many people we should be sending too... 
       my $number_of_users = $IN->param('number_of_fields');
        
       for (my $i = 1; $i <= $number_of_users; $i++) {

          # skip this record if they don't need to be sent too...
           if (!$IN->param("recipemail_$i")) { next; } 

          # first of all, lets check how valid the email address is...
           if (!$IN->param("recipemail_$i") || valid_address($IN->param("recipemail_$i")) == 0) {
             $rec->{'error'} .= "The email address for person $i doesn't seem to be valid....";
             print Links::SiteHTML::display('recommend_it_form', { %$rec, %$IN });  
             exit;
           }

          # now lets check how valid the name is...
           if (!$IN->param("recipname_$i") || length($IN->param("recipname_$i")) < 3) {
             $rec->{'error'} .= "The name you entered for person $i doesn't seem to be valid (or is empty)....";
             print Links::SiteHTML::display('recommend_it_form', { %$rec, %$IN });  
             exit;
           }
          
          # get the list ready, so we can put it in the stats table...
           $SendList .= $IN->param("recipname_$i") . " - " . $IN->param("recipemail_$i") . "<BR>";
          
         # define emails addresses, subjects, etc  
     		  my $TAGS;
    			$TAGS->{'to_email'}   =  $IN->param("recipemail_$i"),
    			$TAGS->{'to_name'}    =  $IN->param("recipname_$i"),
    			$TAGS->{'from_email'} =  $IN->param('senderemail'),
    			$TAGS->{'from_name'}  =  $IN->param('sendername'),
    			$TAGS->{'subject'}    =  $IN->param('subject') || "Site Recommendation...", 

            my $type;
            my $template = "recommend_email_text";
            if ($IN->param('type') eq "html") { 
              $type="text/html";
              $template = "recommend_email_html";
            }

#           my $formatted_message = Links::SiteHTML::display($template, {%$TAGS, %$IN, %$rec});

          Links::send_email($template . ".eml", { %$TAGS, %$IN, %$rec } ) or die "Unable to send message: $GT::Mail::error";

#            GT::Mail->send (
#               smtp	=> $CFG->{db_smtp_server}, 
#               sendmail	=> $CFG->{db_mail_path}, 
#               to	    	=> $TAGS->{'to_email'},
#                from	=> $TAGS->{'from_email'},
#                msg   	=> $formatted_message,
#                subject	=> $TAGS->{'subject'},
#                reply	=> $TAGS->{'from_email'}, 
#                type        => $type,
#                debug	=> $Links::DEBUG
#		 	   ) or Links::fatal ("Unable to send mail: $GT::Mail::error");

        }    

  
}

# ==============================================================================
# Validate email addresses here.. seems to work for pretty much all valid domains :)
# ==============================================================================
sub valid_address {

  my $testmail = $_[0];
  if ($testmail =~ /(@.*@)|(\.\.)|(@\.)|(\.@)|(^\.)/ || $testmail !~ /^.+\@(\[?)[a-zA-Z0-9\-\.]+\.([a-zA-Z]{2,3}|[0-9]{1,3})(\]?)$/) { 
     return 0; 
  } else { 
     return 1; 
  }
  
}
    
1;